package com.winhc.bigdata.flink.func


import com.winhc.bigdata.flink.config.HbaseConfig
import com.winhc.bigdata.flink.java.entity.{PutCollection, SealedPut}
import org.apache.flink.configuration.{ConfigOptions, Configuration}
import org.apache.flink.runtime.state.{FunctionInitializationContext, FunctionSnapshotContext}
import org.apache.flink.streaming.api.checkpoint.CheckpointedFunction
import org.apache.flink.streaming.api.functions.sink.{RichSinkFunction, SinkFunction}
import org.apache.hadoop.hbase.TableName
import org.apache.hadoop.hbase.client.{BufferedMutator, Connection, ConnectionFactory, RetriesExhaustedWithDetailsException}

import java.util.concurrent.atomic.AtomicReference
import scala.collection.JavaConverters.asScalaSetConverter

/**
 * @author: XuJiakai
 * @date: 2021/9/1 17:52
 */
class HbaseSinkFunction() extends RichSinkFunction[PutCollection] with CheckpointedFunction with BufferedMutator.ExceptionListener {
  var connection: Connection = _
  private val failureThrowable: AtomicReference[Throwable] = new AtomicReference[Throwable]

  override def open(parameters: Configuration): Unit = {
    val quorum = ConfigOptions.key("winhc.hbase.config.hbase.zookeeper.quorum").stringType().defaultValue("hb-proxy-pub-uf6m8e1nu4ivp06m5-master1-001.hbase.rds.aliyuncs.com:2181,hb-proxy-pub-uf6m8e1nu4ivp06m5-master2-001.hbase.rds.aliyuncs.com:2181,hb-proxy-pub-uf6m8e1nu4ivp06m5-master3-001.hbase.rds.aliyuncs.com:2181")
    val period = ConfigOptions.key("winhc.hbase.config.hbase.client.scanner.timeout.period").stringType().defaultValue("120000")
    val number = ConfigOptions.key("winhc.hbase.config.hbase.client.retries.number").stringType().defaultValue("5")
    val pause = ConfigOptions.key("winhc.hbase.config.hbase.client.pause").stringType().defaultValue("1000")
    val perserver_tasks = ConfigOptions.key("winhc.hbase.config.hbase.client.max.perserver.tasks").stringType().defaultValue("10")
    val perregion_tasks = ConfigOptions.key("winhc.hbase.config.hbase.client.max.perregion.tasks").stringType().defaultValue("10")
    val maxsize = ConfigOptions.key("winhc.hbase.config.hbase.client.keyvalue.maxsize").stringType().defaultValue("524288000")
    val size = ConfigOptions.key("winhc.hbase.config.base.client.ipc.pool.size").stringType().defaultValue("5")
    val retry = ConfigOptions.key("winhc.hbase.config.zookeeper.recovery.retry").stringType().defaultValue("5")
    val configuration = HbaseConfig.getHbaseConfiguration(parameters, Seq(
      quorum,
      period,
      number,
      pause,
      perserver_tasks,
      perregion_tasks,
      maxsize,
      size,
      retry
    ))
    connection = ConnectionFactory.createConnection(configuration)
  }

  //TODO 按批写入
  override def invoke(value: PutCollection, context: SinkFunction.Context): Unit = {
    value
      .getData
      .entrySet()
      .asScala.map(e => (e.getKey, e.getValue))
      .par
      .foreach(t => {
        val (k, v) = t
        val table = connection.getTable(TableName.valueOf(k))
        table.put(SealedPut.toPuts(v.values()))
      })
  }

  override def close(): Unit = {
    if (connection != null) {
      connection.close()
    }
  }

  private def checkErrorAndRethrow(): Unit = {
    val cause = failureThrowable.get
    if (cause != null) throw new RuntimeException("An error occurred in HbaseSink.", cause)
  }

  override def snapshotState(context: FunctionSnapshotContext): Unit = {
    checkErrorAndRethrow()
  }

  override def initializeState(context: FunctionInitializationContext): Unit = {}

  override def onException(e: RetriesExhaustedWithDetailsException, bufferedMutator: BufferedMutator): Unit = {
    failureThrowable.compareAndSet(null, e)
  }
}
